@@ -31,6 +31,21 @@ sub _compiler_type {
 	  : 'GCC');
 }
 
+sub split_like_shell {
+  # In Win32 command shells, the backslashes in the string "\bar\baz"
+  # should be preserved, because they often show up as parts of
+  # pathnames.  We double-up all these backslashes so shellwords() can
+  # still be used (making sure to avoid backslash-quote pairs, which
+  # should work the same as on Unix).
+
+  (my $self, local $_) = @_;
+  if (defined() && length() && !ref()) {
+    s/\\(?!")/\\\\/g;
+  }
+
+  return $self->SUPER::split_like_shell($_);
+}
+
 sub compile {
   my ($self, %args) = @_;
   my $cf = $self->{config};
@@ -40,6 +55,8 @@ sub compile {
   my ($basename, $srcdir) =
     ( File::Basename::fileparse($args{source}, '\.[^.]+$') )[0,1];
 
+  $srcdir ||= File::Spec->curdir();
+
   my %spec = (
     srcdir      => $srcdir,
     builddir    => $srcdir,
@@ -53,7 +70,7 @@ sub compile {
                    ],
     optimize    => [ $self->split_like_shell($cf->{optimize})    ],
     defines     => [ '' ],
-    includes    => $self->{include_dirs} || [],
+    includes    => $args{include_dirs} || [],
     perlinc     => [
                      File::Spec->catdir($cf->{archlib}, 'CORE'),
                      $self->split_like_shell($cf->{incpath}),
@@ -88,6 +105,7 @@ sub link {
 
   my @objects = ( ref $args{objects} eq 'ARRAY' ? @{$args{objects}} : $args{objects} );
   my $to = join '', (File::Spec->splitpath($objects[0]))[0,1];
+  $to ||= File::Spec->curdir();
 
   (my $file_base = $args{module_name}) =~ s/.*:://;
 
@@ -149,7 +167,7 @@ sub link {
     $self->do_system( @$cmd );
   }
 
-#  (my $out = $spec{output}) =~ tr/'"//d;
+  $spec{output} =~ tr/'"//d;
   return wantarray
     ? grep defined, @spec{qw[output implib explib def_file map_file base_file]}
     : $spec{output};
@@ -210,7 +228,7 @@ sub format_compiler_cmd {
     if $spec{use_scripts};
 
   return [ grep {defined && length} (
-    $spec{cc}, '-c'         ,
+    $spec{cc},'-nologo','-c',
     @{$spec{includes}}      ,
     @{$spec{cflags}}        ,
     @{$spec{optimize}}      ,
